﻿#include <iostream>
#include <opengl-playground/sdlmainwindow.hpp>
#include <opengl-playground/openglshader.hpp>
#include <opengl-playground/textured-rectangle.hpp>

#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 640

int main(int argc, char **argv)
{
	(void)argc;
	(void)argv;
	MainWindow *window;
	GLuint triangle_vao;
	GLuint triangle_vbo;
	SDL_Event event;
	bool run = true;
	bool got_event = false;

	float nav_point[2] = {0.0f, 0.0f};
	float zoom = 1.0;

	float triangle_vertices[] = {	-0.5f, 0.0f,
					0.0f, 0.5f,
					0.0f, 0.0f,
					0.8f, 0.0f,
					0.8f, 0.5f,
					0.5f, 0.2f,
				    };



	window = new SdlMainWindow(WINDOW_HEIGHT, WINDOW_WIDTH);

	window->show();
	window->activateGlContext();

	/* Clear the image and set the background to a nice gray */
	glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
	glClearColor(0.1f, 0.1f, 0.1f, 0.f);
	glClear(GL_COLOR_BUFFER_BIT);

	glGenVertexArrays(1, &triangle_vao);
	glBindVertexArray(triangle_vao);
	glGenBuffers(1, &triangle_vbo);
	glBindBuffer(GL_ARRAY_BUFFER, triangle_vbo);
	glBufferData(GL_ARRAY_BUFFER, sizeof(triangle_vertices), triangle_vertices, GL_STATIC_DRAW);
	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 2 * sizeof(float), (void *)0);
	glEnableVertexAttribArray(0);

	OpenGlShaderProgram prog = OpenGlShaderProgram("shaders/2d-zoom-translate-vertex.glsl", "shaders/triangle-gen-zoomed-geometry.glsl", "shaders/fixed-red-fragment.glsl");
	prog.compile();

	OpenGlShaderProgram textured_rect_prog = OpenGlShaderProgram("shaders/textured-rectangle-vertex.glsl", "", "shaders/textured-rectangle-fragment.glsl");
	textured_rect_prog.compile();

	TexturedRectangle rect = TexturedRectangle(-0.1, -0.1, 0.1, 0.1, textured_rect_prog);
	rect.realize();

	while (run) {
		glClear(GL_COLOR_BUFFER_BIT);
		// Draw the triangles
		prog.use();

		prog.setUniformFloat("zoom", zoom);
		prog.setUniformVec2("center_pos", nav_point);

		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		glBindVertexArray(triangle_vao);
		glDrawArrays(GL_POINTS, 0, sizeof(triangle_vertices)/2);

		rect.setZoom(zoom);
		rect.setOffset(nav_point[0], nav_point[1]);
		rect.render();

		window->swapBuffer();
		got_event = false;

		while (!got_event) {
			if (SDL_WaitEvent(&event)) {
				if (event.type == SDL_KEYDOWN) {
					if (event.key.keysym.sym == SDLK_w) {
						nav_point[1] += 0.05;
					} else if (event.key.keysym.sym == SDLK_s) {
						nav_point[1] -= 0.05;
					} else if (event.key.keysym.sym == SDLK_a) {
						nav_point[0] -= 0.05;
					} else if (event.key.keysym.sym == SDLK_d) {
						nav_point[0] += 0.05;
					} else if (event.key.keysym.sym == SDLK_ESCAPE) {
						std::cout << "Bye!" << std::endl;
						run = false;
					} else if (event.key.keysym.sym == SDLK_PLUS) {
						zoom += 0.01f;
					} else if (event.key.keysym.sym == SDLK_MINUS) {
						zoom -= 0.01f;
						if (zoom < 0.0f)
							zoom = 0.0f;
					}
					got_event = true;

				}
			}
		}


	}

	textured_rect_prog.unload();
	prog.unload();
	delete window;
}
